import re
from typing import Type, TypeVar, Dict
from foundationallm.langchain.exceptions import LangChainException

T = TypeVar('T')  # Generic type variable

class ObjectUtils:

    @staticmethod
    def pascal_to_snake(name):  
        # Convert PascalCase or camelCase to snake_case  
        s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)  
        return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()  

    @staticmethod
    def translate_keys(obj):  
        if isinstance(obj, dict):  
            new_dict = {}  
            for key, value in obj.items():  
                new_key = ObjectUtils.pascal_to_snake(key)  
                new_dict[new_key] = ObjectUtils.translate_keys(value)  # Recursively apply to values  
            return new_dict  
        elif isinstance(obj, list):  
            return [ObjectUtils.translate_keys(item) for item in obj]  # Apply to each item in the list  
        else:  
            return obj  # Return the item itself if it's not a dict or list

    @staticmethod
    def get_object_by_id(object_id: str, objects: dict, object_type: Type[T]) -> T:
        """
        Generic method to retrieve an object of a specified type from a dictionary by its ID.

        Args:
            object_id (str): The ID of the object to retrieve.
            objects (dict): A dictionary containing object data.
            object_type (Type[T]): The type of the object to construct.

        Returns:
            T: An instance of the specified type.

        Raises:
            LangChainException: If the object ID is invalid or the object cannot be constructed.
        """
        if not object_id:
            raise LangChainException("Invalid object ID.", 400)

        object_data = objects.get(object_id)
        if not object_data:
            raise LangChainException(f"Object with ID '{object_id}' not found in the dictionary.", 400)

        try:
            return object_type(**object_data)
        except Exception as e:
            raise LangChainException(f"Failed to construct object of type '{object_type.__name__}': {str(e)}", 400)



from pydantic import BaseModel, Field
from typing import Optional

class FileHistoryItem(BaseModel):
    """
    Represents a file attachment uploaded into the context of a conversation.
    """
    order: int = Field(..., description="The order the file was uploaded in the current conversation.")
    current_message_attachment: Optional[bool] = Field(False, description="Indicates if the file is the attached to the current message.")
    original_file_name: str = Field(..., description="The original file name of the attachment.")
    object_id: str = Field(..., description="The ObjectID of the file attachment resource.")
    file_path: str = Field(..., description="The file path of the attachment in storage.")
    content_type: Optional[str] = Field(None, description="The content type of the attachment.")
    secondary_provider: Optional[str] = Field(None, description="The secondary provider of the attachment.")
    secondary_provider_object_id: Optional[str] = Field(None, description="The identifier of the attachment in the secondary provider.")
    embed_content_in_request: Optional[bool] = Field(False, description="Indicates if the content of the file should be embedded in the request.")



from typing import Optional
from pydantic import BaseModel

class ContentArtifact(BaseModel):
    """
    Source reference information for a completion
    """
    id: str # id in the index
    title: Optional[str] = None # file name
    filepath: Optional[str] = None
    source: Optional[str] = None
    content: Optional[str] = None
    type: Optional[str] = None
    metadata: Optional[dict] = None



from datetime import datetime
from pydantic import Field
from typing import Optional
from foundationallm.models.resource_providers import ResourceName

class ResourceBase(ResourceName):
    """
    Base class containing shared properties for all resources.
    """
    object_id: Optional[str] = Field(default=None, description="The unique identifier of the resource.")
    display_name: Optional[str] = Field(default=None, description="The display name of the resource.")
    description: Optional[str] = Field(default=None, description="The description of the resource.")
    cost_center: Optional[str] = Field(default=None, description="The cost center of the resource.")
    properties: Optional[dict] = Field(default={}, description="The properties of the resource.")
    created_by: Optional[str] = Field(default=None, description="The entity who created the resource.")
    created_on: Optional[datetime] = Field(default=None, description="The date and time the resource was created.")
    updated_by: Optional[str] = Field(default=None, description="The entity who last updated the resource.")
    updated_on: Optional[datetime] = Field(default=None, description="The date and time the resource was last updated.")
    deleted: bool = Field(default=False, description="Indicates if the resource has been logically deleted.")
    expiration_date: Optional[datetime] = Field(default=None, description="The date and time the resource expires.")
    



from enum import Enum

class CompletionRequestObjectKeys(str, Enum):
   """Enumerator of the Completion Request Object Keys."""
   AZUREAI_AGENT_ID = "AzureAI.AgentService.Agent.Id"
   AZUREAI_AGENT_THREAD_ID = "AzureAI.AgentService.Thread.Id"
   OPENAI_ASSISTANT_ID = "OpenAI.Assistants.Assistant.Id"
   OPENAI_THREAD_ID = "OpenAI.Assistants.Thread.Id"
   GATEWAY_API_ENDPOINT_CONFIGURATION = "GatewayAPIEndpointConfiguration"
   INSTANCE_ID = "FoundationaLLM.InstanceId"
   CONTEXT_API_ENDPOINT_CONFIGURATION = "ContextAPIEndpointConfiguration"
   WORKFLOW_INVOCATION_CONVERSATION_FILES = "Workflow.Invocation.ConversationFiles"
   WORKFLOW_INVOCATION_ATTACHED_FILES = "Workflow.Invocation.AttachedFiles"



from pydantic import Field
from typing import Optional
from .message_content_item_base import MessageContentItemBase
from .message_content_item_types import MessageContentItemTypes

class OpenAIFilePathMessageContentItem(MessageContentItemBase):
    """File content item used to generate a message content item."""    
    
    start_index: Optional[int] = Field(None, alias="start_index")
    end_index: Optional[int] = Field(None, alias="end_index")
    file_id: Optional[str] = Field(None, alias="file_id")
    
    def __init__(self, **data):
        super().__init__(**data)
        self.type = MessageContentItemTypes.FILE_PATH



from enum import Enum

class AgentCapabilityCategories(str, Enum):
    """Enumerator of the Agent Capability Categories."""
    AZURE_AI_AGENTS = "AzureAI.AgentService"
    OPENAI_ASSISTANTS = "OpenAI.Assistants"
    FOUNDATIONALLM_KNOWLEDGE_MANAGEMENT = "FoundationaLLM.KnowledgeManagement"
